/* 
 *Database Connect Base Driver Object
 *
 * Author  : Johnathan Ingram
 * Date    : 31 August 1999
 * Updated :
 *   Version 0.2.0 : 16 July 2001, Johnathan Ingram
 */

#ifndef _dbconnBASEDriver
#define _dbconnBASEDriver

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

#define F_NAME_SIZE        256
#define MAX_ERRMSG_SIZE    4096

// ----------------------------------------------------------
// Field object 
// ----------------------------------------------------------

enum DBFieldType {
	FT_NULL,
	FT_BIT, 
	FT_CHAR,
	FT_STRING,
	FT_WSTRING,      //Wide String for unicode.
	FT_BLOB,
	FT_TIMESTAMP,
	FT_DATE,
	FT_TIME,
	FT_DATETIME,
	FT_FLOAT,
	FT_DOUBLE,
	FT_INTEGER,
	FT_SMALLINT,
	FT_WORD,
	FT_BOOLEAN
};


enum conversionType
{
	ct_none,
	ct_from_string,
	ct_from_integer,

	ct_unable
};


class BASE_DBField
{
	public:
		//Field Definition
		char               fieldName[F_NAME_SIZE];
		long int           fieldNum;
		DBFieldType        fieldType;

		bool               isIncrement;
		bool               isPriKey;
		bool               isUnique;
		bool               isNotNull;

		//Defines the current data for the current row.
		bool               isNULL;           //Has no value assigned to the field.
		unsigned long int  dataSize;
		void               *data;
		void               *convertedData;   //Used for storing conversions.
	
		void               *extraInfo;       //Extra information attached to this field object.

		//Constructor / Destructor
		BASE_DBField();
		virtual ~BASE_DBField();

		//Conversion functions
		virtual char*  asString() = 0;
		virtual long   asInteger() = 0;
		virtual double asFloat() = 0;
		virtual time_t asDateTime() = 0;
		virtual bool   asBoolean() = 0;
		virtual void*  asBinary() = 0;
};


// ----------------------------------------------------------
// Base Driver object
// ----------------------------------------------------------

class DBCONNECT_BaseDriver
{
	public:
		// Global Error Message Constants 
		enum DBCON_error_codes 
		{
			DBCON_ERROR_CONNECTING, 
			DBCON_ALREADY_CONNECTED, 
			DBCON_ERROR_DISCONNECTING,
			DBCON_NOT_CONNECTED,
			DBCON_QUERY_ERROR,
			DBCON_FIELD_OUT_OF_RANGE,
			DBCON_DATA_RETRIEVAL_ERROR,
			DBCON_NO_DATA,
			DBCON_EOF_REACHED,
			DBCON_BOF_REACHED,
			DBCON_INVALID_CONVERSION,
			DBCON_TRANSACTION_ALREADY_ACTIVE,
			DBCON_TRANSACTION_NOT_ACTIVE,
			DBCON_TRANSACTION_ERROR
		};
		int  errorCode;                    //Error code.
		char errorMsg[MAX_ERRMSG_SIZE];    //Max size of MAX_ERRMSG_SIZE

		//Dynamic array of fields
		BASE_DBField *fields;

		//Monitoring Flags
		bool               isConnected;
		bool               isBOF;
		bool               isEOF;
		bool               isTransaction;
		bool               errorSet;
		long int           recordNum;    //Current record number.
		long int           recordCount;  //Used for SELECT type statments.
		long int           affectedRows; //Used for INSERT UPDATE type statements.
		long int           lastIdentity; //Used to store the last ID value.
		long int           fieldCount;   //Number of fields in the result set.

		virtual void setError(int globalErrCode, char *info) = 0;
		virtual void clearError() = 0;    
		char* getErrorMsg(int globalErrCode);

		virtual void connect(char *host, char *db_name, char *user, char *password) = 0;
		virtual void disconnect() = 0;

		virtual void execute(char *query) = 0;   
		virtual void first() = 0;
		virtual void last() = 0;
		virtual void next() = 0;
		virtual void prev() = 0;
    
		virtual char* field(unsigned int fieldnum) = 0;

		virtual void transBegin() = 0;
		virtual void transCommit() = 0;
		virtual void transRollback() = 0;
};

#endif
